# $Id:$

package CompartmentList;

use Class::Struct;
use FindBin;
use lib $FindBin::Bin;
use Compartment;
use ParamList;
use strict;

struct CompartmentList => {
  Array=> '@',
  Hash=> '%',
  Used=> '$' # Flag to indicate that compartment attribute is set in 1 or more species in model
};


sub lookup{
  my $clist= shift;
  my $name= shift;
  
  if (defined($clist->Hash->{$name})){
    $clist->Used(1);
    return($clist->Hash->{$name});
  } else {
    return("");
  }
}

sub add{
  my $clist= shift;
  my $name= shift;
  my $spatialDimensions= (@_) ? shift : "";
  my $size= (@_) ? shift : "";
  my $outside= (@_) ? shift : "";

  *hash= $clist->Hash;
  *array= $clist->Array;
  my $comp;

  if (defined($clist->Hash->{$name})){
    # Modify an existing compartment
    $comp= $clist->Hash->{$name};
  } else {
    $comp= Compartment->new;
    $comp->Name($name);
    $clist->Hash->{$name}= $comp;
    push @{$clist->Array}, $comp;
  }
  
  ($spatialDimensions ne "") || return("");
  $comp->SpatialDimensions($spatialDimensions);
  $size && $comp->Size($size);
  $outside && $comp->Outside($outside);
  return("");
}

sub toString{
  my $clist=shift;
  my $string="";
  my $plist=(@_) ? shift : "";

  $string.="begin compartments\n";
  for my $comp (@{$clist->Array}){
    $string.= "  ".$comp->toString($plist)."\n";
  }
  $string.="end compartments\n";

  return($string);
}

# Reads a single compartment specification from a string with the following format:
# name type volume [outside compartments]

sub readString{
  my $clist=shift;
  my $string= shift;
  my $plist= shift;

  my $name;
  if ($string=~s/^\s*([A-Za-z_0-9]+)//){
    $name=$1;
  } else {
    return("Invalid compartment name in $string");
  }
  $clist->add($name);

  my $spatialDimensions;
  if ($string=~s/^\s*(\d+)//){
    $spatialDimensions=$1;
    if ($spatialDimensions<0 || $spatialDimensions>3){
      return("SpatialDimensions set to invalid value $spatialDimensions: must be integer on [0,3]");
    }
  } else {
    return("Invalid Spatial Dimensions for Compartment in $string");
  }

  # Read size
  my $size= Expression->new();
  if (my $err=$size->readString(\$string,$plist)){return("",$err)}

  # Read outside compartment
  my $outside;
  if($string=~s/^([A-Za-z_0-9]+)\s*//){
    my $cname= $1;
    # Get compartment reference
    if (!($outside=$clist->Hash->{$cname})){
      return("Outside compartment $cname is not defined");
    }
  }

  if ($string=~/\S+/){
    return("Unrecognized trailing syntax in compartment specification");
  }

  if (my $err=$clist->add($name, $spatialDimensions, $size, $outside)){
    return($err);
  }

  return("");
}

sub toXML{
  my $clist= shift;
  my $indent=shift;

  my $string=$indent."<ListOfCompartments>\n";
  
  for my $comp (@{$clist->Array}){
    $string.= $comp->toXML("  ".$indent);
  }

  $string.= $indent."</ListOfCompartments>\n";
  return($string);
}

sub validate{
  my $clist=shift;

  for my $comp (@{$clist->Array}){
    if ($comp->SpatialDimensions eq ""){
      my $err= sprintf "Compartment %s referred to but not defined", $comp->Name;
    }
  }
  return("");
}

1;
